@@ -100,12 +100,10 @@ extern int errno;		  /* place where the error numbers go */
 /* The following error codes are generated by the kernel itself. */
 #ifdef _SYSTEM
 #define E_BAD_DEST     -1001	/* destination address illegal */
-#define E_BAD_SRC      -1002	/* source address illegal */
 #define E_TRY_AGAIN    -1003	/* can't send-- tables full */
 #define E_OVERRUN      -1004	/* interrupt for task that is not waiting */
 #define E_BAD_BUF      -1005	/* message buf outside caller's addr space */
 #define E_TASK         -1006	/* can't send to task */
-#define E_NO_MESSAGE   -1007	/* RECEIVE failed: no message present */
 #define E_NO_PERM      -1008	/* ordinary users can't send to tasks */
 #define E_BAD_FCN      -1009	/* only valid fcns are SEND, RECEIVE, BOTH */
 #define E_BAD_ADDR     -1010	/* bad address given to utility routine */
