@@ -1,10 +1,10 @@
-/*
-assert.h
-*/
-#ifndef ASSERT_H
-#define ASSERT_H
+/*	assert.h	*/
 
-#if DEBUG
+#ifndef NDEBUG	/* 8086 must do without training wheels. */
+#define NDEBUG	(_WORD_SIZE == 2)
+#endif
+
+#if !NDEBUG
 
 #define INIT_ASSERT	static char *assert_file= __FILE__;
 
@@ -15,13 +15,11 @@ void bad_compare(char *file, int line, int lhs, char *what, int rhs);
 								: (void) 0)
 #define compare(a,t,b)	(!((a) t (b)) ? bad_compare(assert_file, __LINE__, \
 				(a), #a " " #t " " #b, (b)) : (void) 0)
-#else /* !DEBUG */
+#else /* NDEBUG */
 
 #define INIT_ASSERT	/* nothing */
 
 #define assert(x)	(void)0
 #define compare(a,t,b)	(void)0
 
-#endif /* !DEBUG */
-
-#endif /* ASSERT_H */
+#endif /* NDEBUG */
