@@ -60,6 +60,8 @@ PUBLIC void mem_init()
 }
 #endif /* (CHIP == INTEL) */
 
+PRIVATE char PUNCT[] = ":,;.";
+
 /*=========================================================================*
  *				env_parse				   *
  *=========================================================================*/
@@ -74,12 +76,13 @@ long min, max;		/* minimum and maximum values for the parameter */
  * Panic if the parsing fails.  Return EP_UNSET if the environment variable
  * is not set, EP_OFF if it is set to "off", EP_ON if set to "on" or a
  * field is left blank, or EP_SET if a field is given (return value through
- * *param).  Commas and colons may be used in the environment and format
- * string, fields in the environment string may be empty, and punctuation
- * may be missing to skip fields.  The format string contains characters
- * 'd', 'o', 'x' and 'c' to indicate that 10, 8, 16, or 0 is used as the
- * last argument to strtol.  If the format string contains something like "\4"
- * then the string is repeated at 4 characters left.
+ * *param).  Punctuation may be used in the environment and format string,
+ * fields in the environment string may be empty, and punctuation may be
+ * missing to skip fields.  The format string contains characters 'd', 'o',
+ * 'x' and 'c' to indicate that 10, 8, 16, or 0 is used as the last argument
+ * to strtol().  A '*' means that a field should be skipped.  If the format
+ * string contains something like "\4" then the string is repeated 4 characters
+ * to the left.
  */
 
   char *val, *end;
@@ -98,24 +101,33 @@ long min, max;		/* minimum and maximum values for the parameter */
 
 	if (*fmt == 0) break;		/* too many values */
 
-	if (*val == ',' || *val == ':') {
+	if (strchr(PUNCT, *val) != NULL) {
 		/* Time to go to the next field. */
-		if (*fmt == ',' || *fmt == ':') i++;
+		if (strchr(PUNCT, *fmt) != NULL) i++;
 		if (*fmt++ == *val) val++;
 		if (*fmt < 32) fmt -= *fmt;	/* step back? */
 	} else {
 		/* Environment contains a value, get it. */
 		switch (*fmt) {
+		case '*':	radix =   -1;	break;
 		case 'd':	radix =   10;	break;
 		case 'o':	radix =  010;	break;
 		case 'x':	radix = 0x10;	break;
 		case 'c':	radix =    0;	break;
 		default:	goto badenv;
 		}
-		newpar = strtol(val, &end, radix);
+		
+		if (radix < 0) {
+			/* Skip. */
+			while (strchr(PUNCT, *val) == NULL) val++;
+			continue;
+		} else {
+			/* A number. */
+			newpar = strtol(val, &end, radix);
 
-		if (end == val) break;	/* not a number */
-		val = end;
+			if (end == val) break;	/* not a number */
+			val = end;
+		}
 
 		if (i == field) {
 			/* The field requested. */
@@ -126,11 +138,40 @@ long min, max;		/* minimum and maximum values for the parameter */
 	}
   }
 badenv:
+  env_panic(env);
+}
+
+/*=========================================================================*
+ *				env_panic				   *
+ *=========================================================================*/
+PUBLIC void env_panic(env)
+char *env;		/* environment variable whose value is bogus */
+{
   printf("Bad environment setting: '%s = %s'\n", env, getenv(env));
-  panic(NULL, NO_NUM);
+  panic("", NO_NUM);
   /*NOTREACHED*/
 }
 
+/*=========================================================================*
+ *				env_prefix				   *
+ *=========================================================================*/
+PUBLIC int env_prefix(env, prefix)
+char *env;		/* environment variable to inspect */
+char *prefix;		/* prefix to test for */
+{
+/* An environment setting may be prefixed by a word, usually "pci".  Return
+ * true if a given prefix is used.
+ */
+  char *val;
+  size_t n;
+
+  val = getenv(env);
+  n = strlen(prefix);
+  return(val != NIL_PTR
+	&& strncmp(val, prefix, n) == 0
+	&& strchr(PUNCT, val[n]) != NULL);
+}
+
 #if !NDEBUG
 /*=========================================================================*
  *				bad_assertion				   *
