@@ -16,10 +16,10 @@ Created:	March 15, 1994 by Philip Homburg <philip@cs.vu.nl>
 #define NE2000_START	0x4000
 #define NE2000_SIZE	0x4000
 
-#define inb_ne(dep, reg) (in_byte(dep->de_base_port+reg))
-#define outb_ne(dep, reg, data) (out_byte(dep->de_base_port+reg, data))
-#define inw_ne(dep, reg) (in_word(dep->de_base_port+reg))
-#define outw_ne(dep, reg, data) (out_word(dep->de_base_port+reg, data))
+#define inb_ne(dep, reg)	(inb(dep->de_base_port+reg))
+#define outb_ne(dep, reg, data)	(outb(dep->de_base_port+reg, data))
+#define inw_ne(dep, reg)	(inw(dep->de_base_port+reg))
+#define outw_ne(dep, reg, data)	(outw(dep->de_base_port+reg, data))
 
 #endif /* NE2000_H */
 
