@@ -71,8 +71,8 @@
  * Init pulse length:	   over 200u (not sure).
  *
  * The strobe length is about 50u with the code here and function calls for
- * out_byte() - not much to spare.  The 0.5u minimums may be violated if
- * out_byte() is generated in-line on a fast machine.  Some printer boards
+ * outb() - not much to spare.  The 0.5u minimums may be violated if
+ * outb() is generated in-line on a fast machine.  Some printer boards
  * are slower than 0.5u anyway.
  */
 
@@ -88,6 +88,7 @@ PRIVATE int proc_nr;		/* user requesting the printing */
 PRIVATE int user_left;		/* bytes of output left in user buf */
 PRIVATE vir_bytes user_vir;	/* address of remainder of user buf */
 PRIVATE int writing;		/* nonzero while write is in progress */
+PRIVATE irq_hook_t p_hook;	/* interrupt hook */
 
 FORWARD _PROTOTYPE( void do_cancel, (message *m_ptr) );
 FORWARD _PROTOTYPE( void do_done, (void) );
@@ -96,7 +97,7 @@ FORWARD _PROTOTYPE( void pr_start, (void) );
 FORWARD _PROTOTYPE( void print_init, (void) );
 FORWARD _PROTOTYPE( void reply, (int code, int replyee, int process,
 		int status) );
-FORWARD _PROTOTYPE( int pr_handler, (int irq) );
+FORWARD _PROTOTYPE( int pr_handler, (irq_hook_t *hook) );
 
 /*===========================================================================*
  *				printer_task				     *
@@ -107,12 +108,11 @@ PUBLIC void printer_task()
 
   message pr_mess;		/* buffer for all incoming messages */
 
-  print_init();			/* initialize */
-
   while (TRUE) {
 	receive(ANY, &pr_mess);
 	switch(pr_mess.m_type) {
 	    case DEV_OPEN:
+		print_init();			/* initialize */
 	    case DEV_CLOSE:
 		reply(TASK_REPLY, pr_mess.m_source, pr_mess.PROC_NR, OK);
 		break;
@@ -251,13 +251,17 @@ PRIVATE void print_init()
 /* Set global variables.  Get the port base for the first printer from the
  * BIOS and initialize the printer.
  */
+  static int inited;
+
+  if (inited) return;
+  inited = 1;
 
   phys_copy(0x408L, vir2phys(&port_base), 2L);
-  out_byte(port_base + 2, INIT_PRINTER);
+  outb(port_base + 2, INIT_PRINTER);
   milli_delay(2);		/* easily satisfies Centronics minimum */
-  out_byte(port_base + 2, SELECT);
-  put_irq_handler(PRINTER_IRQ, pr_handler);
-  enable_irq(PRINTER_IRQ);		/* ready for printer interrupts */
+  outb(port_base + 2, SELECT);
+  put_irq_handler(&p_hook, PRINTER_IRQ, pr_handler);
+  enable_irq(&p_hook);		/* ready for printer interrupts */
 }
 
 
@@ -283,8 +287,8 @@ PRIVATE void pr_start()
 /*===========================================================================*
  *				pr_handler				     *
  *===========================================================================*/
-PRIVATE int pr_handler(irq)
-int irq;
+PRIVATE int pr_handler(hook)
+irq_hook_t *hook;
 {
 /* This is the interrupt handler.  When a character has been printed, an
  * interrupt occurs, and the assembly code routine trapped to calls
@@ -306,7 +310,7 @@ int irq;
 	 * when the printer is busy with a previous character, because the
 	 * interrupt status does not affect the printer.
 	 */
-	out_byte(port_base + 2, SELECT);
+	outb(port_base + 2, SELECT);
 	return 1;
   }
 
@@ -314,7 +318,7 @@ int irq;
 	/* Loop to handle fast (buffered) printers.  It is important that
 	 * processor interrupts are not disabled here, just printer interrupts.
 	 */
-	status = in_byte(port_base + 1);
+	status = inb(port_base + 1);
 	if ((status & STATUS_MASK) == BUSY_STATUS) {
 		/* Still busy with last output.  This normally happens
 		 * immediately after doing output to an unbuffered or slow
@@ -326,10 +330,10 @@ int irq;
 	}
 	if ((status & STATUS_MASK) == NORMAL_STATUS) {
 		/* Everything is all right.  Output another character. */
-		out_byte(port_base, *optr++);	/* output character */
+		outb(port_base, *optr++);	/* output character */
 		lock();		/* ensure strobe is not too long */
-		out_byte(port_base + 2, ASSERT_STROBE);
-		out_byte(port_base + 2, NEGATE_STROBE);
+		outb(port_base + 2, ASSERT_STROBE);
+		outb(port_base + 2, NEGATE_STROBE);
 		unlock();
 		opending = FALSE;	/* show interrupt is working */
 
@@ -362,11 +366,11 @@ PUBLIC void pr_restart()
  */
 
   if (oleft != 0) {
-	if (opending && disable_irq(PRINTER_IRQ)) {
-		(void) pr_handler(PRINTER_IRQ);
+	if (opending && disable_irq(&p_hook)) {
+		(void) pr_handler(&p_hook);
 
 		/* ready for printer interrupts again */
-		enable_irq(PRINTER_IRQ);
+		enable_irq(&p_hook);
 	}
 	opending = TRUE;	/* expect some printing before next call */
   }
