@@ -17,13 +17,29 @@ struct memory {
 };
 
 /* Administration for clock polling. */
-struct milli_state {
+struct micro_state {
   unsigned long accum_count;	/* accumulated clock ticks */
   unsigned prev_count;		/* previous clock value */
 };
 
+struct timer;
+typedef void (*tmr_func_t)(struct timer *tp);
+typedef union { int ta_int; void *ta_ptr; } tmr_arg_t;
+
+typedef struct timer
+{
+  struct timer	*tmr_next;	/* next in a timer chain */
+  int		tmr_task;	/* task this timer belongs to */
+  clock_t 	tmr_exp_time;	/* expiry time */
+  tmr_func_t	tmr_func;	/* function to call when expired */
+  tmr_arg_t	tmr_arg;	/* random argument */
+} timer_t;
+
+#define TMR_NEVER LONG_MAX	/* timer not active. */
+
 #if (CHIP == INTEL)
-typedef unsigned port_t;
+typedef u16_t port_t;
+typedef U16_t Port_t;
 typedef unsigned reg_t;		/* machine register */
 
 /* The stack frame layout is determined by the software, but for efficiency
@@ -66,7 +82,14 @@ struct segdesc_s {		/* segment descriptor for protected mode */
   u8_t base_high;
 };
 
-typedef _PROTOTYPE( int (*irq_handler_t), (int irq) );
+typedef struct irq_hook {
+  struct irq_hook *next;
+  int (*handler)(struct irq_hook *);
+  int irq;
+  int id;
+} irq_hook_t;
+
+typedef int (*irq_handler_t)(struct irq_hook *);
 
 #endif /* (CHIP == INTEL) */
 
