@@ -126,7 +126,7 @@ FORWARD _PROTOTYPE( int w_1rdwt, (int opcode, unsigned long block,
 FORWARD _PROTOTYPE( int win_results, (void) );
 FORWARD _PROTOTYPE( void win_out, (int val) );
 FORWARD _PROTOTYPE( int w_reset, (void) );
-FORWARD _PROTOTYPE( int w_handler, (int irq) );
+FORWARD _PROTOTYPE( int w_handler, (irq_hook_t hook) );
 FORWARD _PROTOTYPE( int win_specify, (int drive) );
 FORWARD _PROTOTYPE( int check_init, (void) );
 FORWARD _PROTOTYPE( int read_ecc, (void) );
@@ -158,12 +158,14 @@ PRIVATE struct driver w_dtab = {
  *===========================================================================*/
 PUBLIC void xt_winchester_task()
 {
+  static irq_hook_t hook;
+
   win_tasknr = proc_number(proc_ptr);
 
   init_params();
 
-  put_irq_handler(XT_WINI_IRQ, w_handler);
-  enable_irq(XT_WINI_IRQ);	/* ready for winchester interrupts */
+  put_irq_handler(&hook, XT_WINI_IRQ, w_handler);
+  enable_irq(&hook);	/* ready for winchester interrupts */
 
   driver_task(&w_dtab);
 }
@@ -380,13 +382,13 @@ unsigned count;			/* bytes to transfer */
  */
 
   /* Set up the DMA registers. */
-  out_byte(DMA_FLIPFLOP, 0);		/* write anything to reset it */
-  out_byte(DMA_MODE, opcode == DEV_SCATTER ? DMA_WRITE : DMA_READ);
-  out_byte(DMA_ADDR, (int) address >>  0);
-  out_byte(DMA_ADDR, (int) address >>  8);
-  out_byte(DMA_TOP, (int) (address >> 16));
-  out_byte(DMA_COUNT, (count - 1) >> 0);
-  out_byte(DMA_COUNT, (count - 1) >> 8);
+  outb(DMA_FLIPFLOP, 0);		/* write anything to reset it */
+  outb(DMA_MODE, opcode == DEV_SCATTER ? DMA_WRITE : DMA_READ);
+  outb(DMA_ADDR, (int) address >>  0);
+  outb(DMA_ADDR, (int) address >>  8);
+  outb(DMA_TOP, (int) (address >> 16));
+  outb(DMA_COUNT, (count - 1) >> 0);
+  outb(DMA_COUNT, (count - 1) >> 8);
 }
 
 
@@ -420,7 +422,7 @@ unsigned int count;		/* bytes to transfer */
   if (com_out(DMA_INT, command) != OK)
 	return(ERR);
 
-  out_byte(DMA_INIT, 3);	/* initialize DMA */
+  outb(DMA_INIT, 3);	/* initialize DMA */
 
   /* Block, waiting for disk interrupt. */
   receive(HARDWARE, &mess);
@@ -446,8 +448,8 @@ PRIVATE int win_results()
   int i, status;
   u8_t command[6];
 
-  status = in_byte(WIN_DATA);
-  out_byte(WIN_DMA, 0);
+  status = inb(WIN_DATA);
+  outb(WIN_DMA, 0);
   if (!(status & 2))		/* Test "error" bit */
 	return(OK);
   command[0] = WIN_SENSE;
@@ -459,13 +461,13 @@ PRIVATE int win_results()
   for (i = 0; i < MAX_RESULTS; i++) {
 	if (hd_wait(WST_REQ) != OK)
 		return(ERR);
-	status = in_byte(WIN_DATA);
+	status = inb(WIN_DATA);
 	w_results[i] = status & BYTE;
   }
   if (hd_wait(WST_REQ) != OK)	/* Missing from			*/
 	 return (ERR);		/* Original.  11-Apr-87 G.O.	*/
 
-  status = in_byte(WIN_DATA);	 /* Read "error" flag */
+  status = inb(WIN_DATA);	 /* Read "error" flag */
 
   if (((status & 2) != 0) || (w_results[0] & 0x3F)) {
 	return(ERR);
@@ -489,10 +491,10 @@ int val;		/* write this byte to winchester disk controller */
   if (w_need_reset) return;	/* if controller is not listening, return */
 
   do {
-	r = in_byte(WIN_STATUS);
+	r = inb(WIN_STATUS);
   } while((r & (WST_REQ | WST_BUSY)) == WST_BUSY);
 
-  out_byte(WIN_DATA, val);
+  outb(WIN_DATA, val);
 }
 
 
@@ -510,13 +512,13 @@ PRIVATE int w_reset()
   message mess;
 
   /* Strobe reset bit low. */
-  out_byte(WIN_STATUS, 0);
+  outb(WIN_STATUS, 0);
 
   milli_delay(5);	/* Wait for a while */
 
-  out_byte(WIN_SELECT, 0);	/* Issue select pulse */
+  outb(WIN_SELECT, 0);	/* Issue select pulse */
   for (i = 0; i < MAX_WIN_RETRY; i++) {
-	r = in_byte(WIN_STATUS);
+	r = inb(WIN_STATUS);
 	if (r & (WST_DRQ | WST_IRQ))
 		return(ERR);
 
@@ -558,17 +560,17 @@ PRIVATE int w_reset()
 /*==========================================================================*
  *				w_handler				    *
  *==========================================================================*/
-PRIVATE int w_handler(irq)
-int irq;
+PRIVATE int w_handler(hook)
+irq_hook_t *hook;
 {
 /* Disk interrupt, send message to winchester task and reenable interrupts. */
 
   int r, i;
 
-  out_byte(DMA_INIT, 0x07);	/* Disable int from DMA */
+  outb(DMA_INIT, 0x07);		/* Disable int from DMA */
 
   for (i = 0; i < MAX_WIN_RETRY; ++i) {
-	r = in_byte(WIN_STATUS);
+	r = inb(WIN_STATUS);
 	if (r & WST_IRQ)
 		break;		/* Exit if end of int */
   }
@@ -634,10 +636,10 @@ PRIVATE int check_init()
   int r, s;
 
   if (hd_wait(WST_REQ | WST_INPUT) == OK) {
-	r = in_byte(WIN_DATA);
+	r = inb(WIN_DATA);
 
 	do {
-		s = in_byte(WIN_STATUS);
+		s = inb(WIN_STATUS);
 	} while(s & WST_BUSY);		/* Loop while still busy */
 
 	if (r & 2)		/* Test error bit */
@@ -661,9 +663,9 @@ PRIVATE int read_ecc()
 
   command[0] = WIN_ECC_READ;
   if (com_out(NO_DMA_INT, command) == OK && hd_wait(WST_REQ) == OK) {
-	r = in_byte(WIN_DATA);
+	r = inb(WIN_DATA);
 	if (hd_wait(WST_REQ) == OK) {
-		r = in_byte(WIN_DATA);
+		r = inb(WIN_DATA);
 		if (r & 1)
 			w_need_reset = TRUE;
 	}
@@ -683,7 +685,7 @@ int bits;
   int r, i = 0;
 
   do {
-	r = in_byte(WIN_STATUS) & bits;
+	r = inb(WIN_STATUS) & bits;
   } while ((i++ < MAX_WIN_RETRY) && r != bits);		/* Wait for ALL bits */
 
   if (i >= MAX_WIN_RETRY) {
@@ -705,10 +707,10 @@ u8_t *commandp;
 
   int i, r;
 
-  out_byte(WIN_DMA, mode);
-  out_byte(WIN_SELECT, mode);
+  outb(WIN_DMA, mode);
+  outb(WIN_SELECT, mode);
   for (i = 0; i < MAX_WIN_RETRY; i++) {
-	r = in_byte(WIN_STATUS);
+	r = inb(WIN_STATUS);
 	if (r & WST_BUSY)
 		break;
   }
@@ -723,13 +725,13 @@ u8_t *commandp;
 	if (hd_wait(WST_REQ) != OK)
 		break;		/* No data request pending */
 
-	r = in_byte(WIN_STATUS);
+	r = inb(WIN_STATUS);
 
 	if ((r & (WST_BUSY | WST_BUS | WST_INPUT)) !=
 		(WST_BUSY | WST_BUS))
 		break;
 
-	out_byte(WIN_DATA, commandp[i]);
+	outb(WIN_DATA, commandp[i]);
   }
 
   if (i != 6)
@@ -760,7 +762,7 @@ PRIVATE void init_params()
   if (nr_drives > MAX_DRIVES) nr_drives = MAX_DRIVES;
 
   /* Read the switches from the controller */
-  w_switches = in_byte(WIN_SELECT);
+  w_switches = inb(WIN_SELECT);
 
 #if AUTO_BIOS
   /* If no auto configuration or not enabled then go to the ROM. */
